
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from collections import deque

class DiscoveryManager(object):
    
    def __init__(self, core):
        self._core = core
        self._request_list = deque()
        #self._response_list = deque()
        self._done_requests = set()
        self._finger_plugins = None
        self._filter_plugins = None
        self._consumer_plugins = None
    
    def start(self):
        
        #First phase, execute finger plugins and extend request list
        for plugin in self._finger_plugins:
            self._request_list.extend(plugin.start())
        while self._request_list:
            request_todo = self._request_list.popleft()
            if request_todo in self._done_requests:
                continue
            self._done_requests.add(request_todo)
            response = self._send(request_todo)
            if response:
                if response.get_code() == 200:
                    self._core.get_to_audit().append(request_todo)
                for plugin in self._consumer_plugins:
                    self._request_list.extend(plugin.consume(response))
    
    def _send(self, request):
        return self._filter_plugins[0].filter(request)
    
    def next(self, request, plugin):
        index = self._filter_plugins.index(plugin)
        return self._filter_plugins[index+1].filter(request)
    
    def set_finger_plugins(self, plugins):
        self._finger_plugins = plugins
    
    def set_filter_plugins(self, plugins):
        self._filter_plugins = plugins
        self._filter_plugins.sort(key=lambda x: x.get_weight())
    
    def set_consumer_plugins(self, plugins):
        self._consumer_plugins = plugins
    
    def clean(self):
        for plugin in self._finger_plugins:
            plugin.clean()
        for plugin in self._consumer_plugins:
            plugin.clean()
        for plugin in self._filter_plugins:
            plugin.clean()
        self._finger_plugins = None
        self._consumer_plugins = None
        self._filter_plugins = None
        self._done_requests = set()
        self._request_list = deque()
    