
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import copy
import hashlib

class Request(object):
    
    def __init__(self, method, url, parameters = {}, headers = {}):
        self._method = method
        self._url = url
        self._parameters = parameters
        self._headers = headers
    
    def clone(self):
        return copy.deepcopy(self)
    
    def get_method(self):
        return self._method
    
    def get_url(self):
        return self._url
    
    def get_parameters(self):
        return copy.deepcopy(self._parameters)
    
    def get_headers(self):
        return copy.deepcopy(self._headers)
    
    def __eq__(self, another_request):
        return self._url == another_request.get_url() and \
        self._method == another_request.get_method() and \
        self._parameters == another_request.get_parameters() and \
        self._headers == another_request.get_headers()
    
    def __hash__(self):
        base_string = self._method + self._url + str(self._parameters) + str(self._headers)
        hash = hashlib.md5(base_string)
        return int(hash.hexdigest(), 16)
