#!/usr/bin/python
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import web
import sys
import os
import threading

import BeautifulSoup

render = web.template.render('./frontends/web/templates/')

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
os.chdir(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
import core.RabbitCore
from utils.ClassFactory import class_from_name

urls = ('/', 'index',
        '/update/(.*)', 'update_plugins',
        '/session', 'session',
        '/plugins/(.*)/(.*)', 'plugin_options',
        '/scan', 'scan',
        '/scan/info', 'scan_info')

thread = None

def check_scanning():
    global thread
    if thread and thread.is_alive():
        raise web.seeother('/scan/info')


class scan_info:
    
    def GET(self):
        if 'HTMLOutput' in rabbit_core.get_option('output_plugins').get_value():
            cl = class_from_name("plugins.output.HTMLOutput")
            f = open(cl.get_class_option('filename').get_value(), "r")
            html = f.read()
            f.close()
            parsed_report = BeautifulSoup.BeautifulSoup("<div class=\"report\"></div>")
            div = parsed_report.find('div')
            bsoup = BeautifulSoup.BeautifulSoup(html)
            i = 0
            try:
                for child in bsoup.body:
                    if isinstance(child, BeautifulSoup.NavigableString):
                        continue
                    div.insert(i, child)
                    i += 1
                parsed_report = parsed_report.prettify()
            except TypeError, e:
                parsed_report = ''
        else:
            parsed_report = ''
        return render.scan_info(rabbit_core.get_state(), parsed_report)

class scan:
    
    def GET(self):
        global thread
        check_scanning()
        thread = threading.Thread(target=rabbit_core.start)
        thread.start()
        raise web.seeother('/scan/info')

class plugin_options:
    
    def GET(self, type, plugin_name):
        check_scanning()
        full_plugin_name = 'plugins.' + type.replace('-', '.') + '.' + plugin_name
        try:
            plugin_class = class_from_name(full_plugin_name)
        except ImportError, e:
            return render.error(str(e))
        return render.plugin_detail(plugin_name, plugin_class, plugin_class.get_class_options())
    
    def POST(self, type, plugin_name):
        check_scanning()
        input = web.input()
        full_plugin_name = 'plugins.' + type.replace('-', '.') + '.' + plugin_name
        plugin_class = class_from_name(full_plugin_name)
        for option in plugin_class.get_class_options():
            if option.get_variable() not in input:
                continue
            if option.get_value_class() == 'list_str':
                value = [val.strip() for val in input[option.get_variable()].split(',')]
            elif option.get_value_class() == 'list_int':
                value = [int(val.strip()) for val in input[option.get_variable()].split(',')]
            elif option.get_value_class() == 'int':
                value = int(input[option.get_variable()])
            else:
                value = input[option.get_variable()].strip()
            option.set_value(value)
        return render.plugin_detail(plugin_name, plugin_class, plugin_class.get_class_options())

class session:
    
    def POST(self):
        check_scanning()
        i = web.input()
        if i['action'] == 'Save':
            rabbit_core.get_config_manager().save(i['session_name'])
        elif i['action'] == 'Load':
            rabbit_core.get_config_manager().load(i['session_name'])
        else:
            return render.error("The action you are trying to do doesn't exist")
        raise web.seeother('/')
    
class index:
    
    def GET(self):
        check_scanning()
        session = rabbit_core.get_config_manager().get_option('session').get_value()
        
        discovery_finger_plugins = [x[:-3] for x in os.listdir("./plugins/discovery/finger/") if not x.startswith('__') and x.endswith('.py')]
        discovery_finger_plugins = [(x, x in rabbit_core.get_option('finger_plugins').get_value() and "CHECKED" or "") for x in discovery_finger_plugins]
        
        discovery_filter_plugins = [x[:-3] for x in os.listdir("./plugins/discovery/filter/") if not x.startswith('__') and x.endswith('.py')]
        discovery_filter_plugins = [(x, x in rabbit_core.get_option('discovery_filter_plugins').get_value() and "CHECKED" or "") for x in discovery_filter_plugins]
        
        discovery_information_plugins = [x[:-3] for x in os.listdir("./plugins/discovery/information_finder/") if not x.startswith('__') and x.endswith('.py')]
        discovery_information_plugins = [(x, x in rabbit_core.get_option('information_finder_plugins').get_value() and "CHECKED" or "") for x in discovery_information_plugins]
        
        discovery_request_plugins = [x[:-3] for x in os.listdir("./plugins/discovery/request_generator/") if not x.startswith('__') and x.endswith('.py')]
        discovery_request_plugins = [(x, x in rabbit_core.get_option('request_generator_plugins').get_value() and "CHECKED" or "") for x in discovery_request_plugins]
        
        output_plugins = [x[:-3] for x in os.listdir("./plugins/output/") if not x.startswith('__') and x.endswith('.py')]
        output_plugins = [(x, x in rabbit_core.get_option('output_plugins').get_value() and "CHECKED" or "") for x in output_plugins]
        
        request_filter_plugins = [x[:-3] for x in os.listdir("./plugins/request/filter/") if not x.startswith('__') and x.endswith('.py')]
        request_filter_plugins = [(x, x in rabbit_core.get_option('request_filter_plugins').get_value() and "CHECKED" or "") for x in request_filter_plugins]
        
        audit_plugins = [x[:-3] for x in os.listdir("./plugins/audit/") if not x.startswith('__') and x.endswith('.py')]
        audit_plugins = [(x, x in rabbit_core.get_option('audit_plugins').get_value() and "CHECKED" or "") for x in audit_plugins]
        
        return render.index(session,
                            discovery_finger_plugins,
                            discovery_filter_plugins,
                            discovery_information_plugins,
                            discovery_request_plugins,
                            output_plugins,
                            request_filter_plugins,
                            audit_plugins)

class update_plugins:
    
    def POST(self, option):
        check_scanning()
        i = web.input()
        new_value = [key for key in i if i[key] == u'True']
        if option not in [opt.get_variable() for opt in rabbit_core.get_options()]:
            return render.error('Not valid option.')
        rabbit_core.get_option(option).set_value(new_value)
        raise web.seeother('/')

if __name__ == '__main__':
    app = web.application(urls, globals())
    app.run()

