
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#    This plugin was written by Santiago Alessandri, CERT.UNLP(salessandri@cert.unlp.edu.ar)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from hashlib import md5
import random
import re

from core.audit_subsystem.AuditPlugin import AuditPlugin
from core.request_subsystem.Request import Request
from core.audit_subsystem.Vulnerability import Vulnerability
import core.constants.dbms as dbms
import core.constants.severities as severity
from core.constants.vulnerabilities import LFI

class LocalFileInclusion(AuditPlugin):
    
    def __init__(self):
        AuditPlugin.__init__(self)
        self._error_injection_file = md5(str(random.randint(0, 1500))).hexdigest().upper()
        self._error_messages = ("<b>Warning</b>:\s*include(_once)?\(\)\s*\[.*function.include.*\]:\s*Failed\sopening\s*'(.*?)'\sfor\sinclusion",
                                "<b>Fatal\s*error</b>:\s*require(_once)?\(\)\s*\[.*function.require.*\]:\s*Failed\sopening\srequired\s*'(.*?)'")
        self._error_messages = map(re.compile, self._error_messages)
        self._common_files = (('../' * 20 + 'etc/passwd\0', re.compile('root:.*?:0:0:')),
                              ('../' * 20 + 'proc/self/environ\0', re.compile('PATH=')),
                              ('../' * 20 + 'boot.ini\0', re.compile('\[boot loader\]')),
                              )
    
    def audit(self, request):
        
        for parameter in request.get_parameters():
            if self._try_error_output(request, parameter):
                continue
            if self._try_common_files(request, parameter):
                continue
    
    def _try_error_output(self, request, parameter):
        new_params = request.get_parameters()
        new_params[parameter] = self._error_injection_file
        new_request = Request(request.get_method(),
                              request.get_url(),
                              new_params,
                              request.get_headers())
        response = self._send(new_request)
        if response == None:
            return False
        for regex in self._error_messages:
            regex_search_res = regex.search(response.get_content())
            #self.get_core().get_output_manager().debug("LFI Plugin Trying in url: %s --> %s(%s)\nContent found: %s" % (request.get_url(), parameter, self._error_injection_file, response.get_content()))
            if regex_search_res and self._error_injection_file in regex_search_res.groups()[1].upper():
                #Local File Inclusion found for error strings
                vuln = Vulnerability(response.get_request(), LFI, severity.HIGH, parameter)
                self.get_core().get_output_manager().vulnerability(vuln)
                return True
        return False
    
    def _try_common_files(self, request, parameter):
        
        orig_response = self._send(request)
        if orig_response == None:
            return False
        
        new_params = request.get_parameters()
        new_params[parameter] = './' + new_params[parameter]
        new_request = Request(request.get_method(),
                              request.get_url(),
                              new_params,
                              request.get_headers())
        response = self._send(new_request)
        if response == None:
            return False
        
        if orig_response.get_content().replace('./', '') != response.get_content().replace('./', ''):
            return False
        
        for to_inject, expect in self._common_files:
            new_params = request.get_parameters()
            new_params[parameter] = to_inject
            new_request = Request(request.get_method(),
                                  request.get_url(),
                                  new_params,
                                  request.get_headers())
            response = self._send(new_request)
            
            if response == None:
                return False
            if expect.search(response.get_content()):
                #Local File Inclusion found including common files
                vuln = Vulnerability(response.get_request(), LFI, severity.HIGH, parameter)
                self.get_core().get_output_manager().vulnerability(vuln)
                return True
        return False
    