
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#    This plugin was written by Santiago Alessandri, CERT.UNLP(salessandri@cert.unlp.edu.ar)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from hashlib import md5
import random
import re

from core.audit_subsystem.AuditPlugin import AuditPlugin
from core.request_subsystem.Request import Request
from core.audit_subsystem.Vulnerability import Vulnerability
import core.constants.severities as severity
from core.constants.vulnerabilities import RFI

class RemoteFileInclusion(AuditPlugin):
    
    def __init__(self):
        AuditPlugin.__init__(self)
        self._inclusion_strings = ('http://rabbit-vs.sourceforge.net/hashes.txt',
                                   'http://rabbit-vs.sourceforge.net/hashes.txt?parameter=')
        self._content = None
    
    def audit(self, request):
        if self._content  is None:
            r = self._send(Request('GET', 'http://rabbit-vs.sourceforge.net/hashes.txt'))
            self._content = r.get_content()
            
        for parameter in request.get_parameters():
            if self._try_inclusion_strings(request, parameter):
                continue
    
    def _try_inclusion_strings(self, request, parameter):
        
        for to_inject in self._inclusion_strings:
            new_params = request.get_parameters()
            new_params[parameter] = to_inject
            new_request = Request(request.get_method(),
                                  request.get_url(),
                                  new_params,
                                  request.get_headers())
            response = self._send(new_request)
            
            if response == None:
                return False
            if response.get_content().find(self._content) > -1:
                #Remote File Inclusion found.
                vuln = Vulnerability(response.get_request(), RFI, severity.HIGH, parameter)
                self.get_core().get_output_manager().vulnerability(vuln)
                return True
        return False
    