
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import os

import xml.dom.minidom
import pickle

from core.config_subsystem.ConfigPlugin import ConfigPlugin
from core.constants.option_types import STRING

class XMLConfigPlugin(ConfigPlugin):
    
    def __init__(self):
        ConfigPlugin.__init__(self)
        self._session_path = os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))) + '/etc/XMLConfigPlugin/'
    
    def save(self, session_name, categories):
        dom_tree = xml.dom.minidom.Document()
        
        root_node = dom_tree.createElement("config")
        
        for category in categories:
            category_node = dom_tree.createElement(category)
            
            variables = categories[category]
            for variable in variables:
                variable_node = dom_tree.createElement(variable)
                
                class_name = variables[variable].get_value_class()
                variable_node.setAttribute("class", class_name)
                variable_node.setAttribute("value", pickle.dumps(variables[variable].get_value()).encode('base64').replace('\n', ''))
                
                category_node.appendChild(variable_node)
            
            root_node.appendChild(category_node)
        
        dom_tree.appendChild(root_node)
        
        fd = open(self._session_path + session_name + ".xml", "w")
        fd.write(dom_tree.toprettyxml(indent="    "))
        fd.close()
    
    def load(self, session_name):
        dom_tree = xml.dom.minidom.parse(self._session_path + session_name + ".xml").firstChild
        
        categories = dom_tree.childNodes
        for category in categories:
            if category.nodeName == "#text":
                continue
            variables = category.childNodes
            for variable in variables:
                if variable.nodeName == "#text":
                    continue
                type  = variable.getAttribute("class")
                value = pickle.loads(variable.getAttribute("value").decode('base64'))
                option = self.create_option(category.nodeName, variable.nodeName, value, type)
                option.set_value(value)
