###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import re
import urlparse

from core.request_subsystem.Request import Request
from core.discovery_subsystem.RequestGeneratorPlugin import RequestGeneratorPlugin

class CSSSpiderPlugin(RequestGeneratorPlugin):
    
    def __init__(self):
        RequestGeneratorPlugin.__init__(self)
    
    def _get_url_path(self, response, uri):
        
        url = urlparse.urljoin(response.get_request().get_url(), uri)
        
        parsed_uri = list(urlparse.urlsplit(url))
        
        query = urlparse.parse_qs(parsed_uri[3])
        params = {}
        for q in query:
            item = query[q][0]
            params[q] = item
        
        parsed_uri[3] = ''
        parsed_uri[4] = ''
        
        url = urlparse.urlunsplit(parsed_uri)
        
        return (url, params)

    def consume(self, response):
        requests = []
        
        '''
            No necesariamente existe el campo content-type con text/css, pero es
            algo que por ahora asumimos de esa manera.
        '''
        if response.get_headers()['CONTENT-TYPE'] != 'text/css':
            return requests
        
        uris = re.findall("url\((.+)\)", response.get_content())
        
        for uri in uris:
            url, params = self._get_url_path(response, uri)
            requests.append(Request("GET", url, params))
        
        return requests





