###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import HTMLParser
from BeautifulSoup import BeautifulSoup
import urlparse

from core.request_subsystem.Request import Request
from core.discovery_subsystem.RequestGeneratorPlugin import RequestGeneratorPlugin

class HTMLSpiderPlugin(RequestGeneratorPlugin):
    
    def __init__(self):
        self._interesting_tags = {'a':True,
                                  'form':True,
                                  'link':True,
                                  'img':True,
                                  'script':True,
                                  'iframe':True,
                                  'frame':True}
        RequestGeneratorPlugin.__init__(self)
        
        '''
            Con este diccionario, despachamos la responsabilidad del procesamiento
            de cada uno de los posibles tags a metodos especializados.
        '''
        self.tag_info_getters = {'a':self._href_getter,
                                 'form':self._action_getter,
                                 'link':self._href_getter,
                                 'img':self._src_getter,
                                 'script':self._src_getter,
                                 'iframe':self._src_getter,
                                 'frame':self._src_getter}
    
    def _href_getter(self, bs_tag):
        try:
            uri = bs_tag['href']
        except  KeyError, e:
            return None
        
        url, params = self._generate_request_info(self.response, uri)
        return Request('GET', url, params)
    
    def _get_param_from_input(self, bs_tag):
        know_types = ['checkbox',
                      'hidden',
                      'password',
                      'radio',
                      'text']
        try:
            if bs_tag['type'] in know_types:
                name = bs_tag['name']
            else:
                return (None, None)
        except KeyError, e:
            return (None, None)
        
        try:
            value = bs_tag['value']
        except KeyError, e:
            value = ''
        
        return (name, value)
    
    def _action_getter(self, bs_tag):
        try:
            uri = bs_tag['action']
        except KeyError, e:
            uri = ''
        
        params = {}
        input_tags = bs_tag.findAll('input')
        for input in input_tags:
            name, value = self._get_param_from_input(input)
            if name != None:
                params[name] = value
        
        try:
            method = bs_tag['method']
        except KeyError, e:
            method = 'GET'
        
        url, url_params = self._generate_request_info(self.response, uri)
        params.update(url_params)
        return Request(method, url, params)
    
    def _src_getter(self, bs_tag):
        try:
            uri = bs_tag['src']
        except  KeyError, e:
            return None
        
        url, params = self._generate_request_info(self.response, uri)
        return Request('GET', url, params)
    
    def _generate_request_info(self, response, uri):
        
        url = urlparse.urljoin(response.get_request().get_url(), uri)
        
        parsed_uri = list(urlparse.urlsplit(url))
        
        query = urlparse.parse_qs(parsed_uri[3])
        params = {}
        for q in query:
            item = query[q][0]
            params[q] = item
        
        parsed_uri[3] = ''
        parsed_uri[4] = ''
        
        url = urlparse.urlunsplit(parsed_uri)
        
        return (url, params)
    
    def consume(self, response):
        
        '''
            TODO: Deberiamos controlar que el content-type sea text/html
        '''
        
        requests = []
        self.response = response
        
        '''
            Obtenemos todos los tags HTML que nos puedan parecer interesantes
            y obtenemos la informacion de ellos.
        '''
        try:
            code_soup = BeautifulSoup(response.get_content())
        except HTMLParser.HTMLParseError:
            return []
        tags = code_soup.findAll(self._interesting_tags)
        
        for tag in tags:
            '''
                TODO:Lo ponemos de esta manera para que sea facil poder agregar
                nuevos tags a buscar.
            '''
            
            if not tag.name in self.tag_info_getters:
                continue
            
            tmp_request = self.tag_info_getters[tag.name](tag)
            
            if tmp_request != None:
                requests.append(tmp_request)
        
        return requests
