
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import datetime
from threading import Lock

from core.output_subsystem.OutputPlugin import OutputPlugin
from core.constants.option_types import STRING

class TextOutput(OutputPlugin):
    
    options = (
        OutputPlugin.create_option('output.TextOutput', 'filename', 'file.out', STRING),
    )
    
    def __init__(self):
        OutputPlugin.__init__(self)
        self._file = open(self.get_option('filename').get_value(), "w")
        self._mutex = Lock()
    
    def debug(self, message):
        msg = "[%s][%15s]:%s\n" % (str(datetime.datetime.now()), "DEBUG", message)
        with self._mutex:
            self._file.write(msg)
    
    def information(self, message):
        msg = "[%s][%15s]:%s\n" % (str(datetime.datetime.now()), "INFORMATION", message)
        with self._mutex:
            self._file.write(msg)
    
    def error(self, message):
        msg = "[%s][%15s]:%s\n" % (str(datetime.datetime.now()), "ERROR", message)
        with self._mutex:
            self._file.write(msg)
    
    def log(self, message):
        msg = "[%s][%15s]:%s\n" % (str(datetime.datetime.now()), "LOG", message)
        with self._mutex:
            self._file.write(msg)
    
    def vulnerability(self, vuln):
        msg = "[%s][%15s]:%s\n" % (str(datetime.datetime.now()), "VULNERABILITY", str(vuln))
        with self._mutex:
            self._file.write(msg)
    
    def clean(self):
        self._file.close()
    
    