
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from core.request_subsystem.RequestFilterPlugin import RequestFilterPlugin
from core.request_subsystem.Request import Request
from core.constants.option_types import STRING

class BasicAuthPlugin(RequestFilterPlugin):
    
    weight = 2 ** 31 - 3
    
    options = (
        RequestFilterPlugin.create_option('request.filter.BasicAuthPlugin', 'passwd_file', 'htaccess', STRING),
    )
    
    def __init__(self):
        RequestFilterPlugin.__init__(self)
        passwd_file = open(self.get_option('passwd_file').get_value())
        self._passwd = passwd_file.readlines()
        self._passwd = [line.strip() for line in self._passwd]
        passwd_file.close()
    
    def request(self, request):
        response = self._next(request)
        if response and response.get_code() == 401:
            for user in self._passwd:
                new_headers = request.get_headers()
                new_headers['Authorization'] = 'Basic ' + user.encode('base64').replace('\n', '')
                new_request = Request(request.get_method(),
                                      request.get_url(),
                                      request.get_parameters(),
                                      new_headers)
                response = self._next(new_request)
                if response == None or response.get_code() != 401:
                    break
        return response