
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import re
import urlparse

from Cookie import SimpleCookie, CookieError
from core.request_subsystem.RequestFilterPlugin import RequestFilterPlugin
from core.request_subsystem.Request import Request
from core.constants.option_types import STRING, LIST_STR

class CookieCollectorSender(RequestFilterPlugin):
    
    weight = 2 ** 31 - 2
    
    options = (
        RequestFilterPlugin.create_option('request.filter.CookieCollectorSender', 'default_cookie', None, STRING),
        RequestFilterPlugin.create_option('request.filter.CookieCollectorSender', 'cookie_headers', ['COOKIE', 'COOKIE2', 'SET-COOKIE'], LIST_STR),
    )
    
    def __init__(self):
        RequestFilterPlugin.__init__(self)
        self.get_core().get_knowledge_base().set('request.filter.CookieCollectorSender', 'cookies', {})
        self._cookie_headers = self.get_option('cookie_headers').get_value()
    
    def request(self, request):
        host = urlparse.urlparse(request.get_url()).netloc
        cookie_dict = self.get_core().get_knowledge_base().get('request.filter.CookieCollectorSender', 'cookies')
        if host in cookie_dict and 'COOKIE' not in request.get_headers():
            new_headers = request.get_headers()
            new_headers['COOKIE'] = ";".join([v.OutputString() for v in cookie_dict[host].values()])
            request = Request(request.get_method(),
                              request.get_url(),
                              request.get_parameters(),
                              new_headers)
        response = self._next(request)
        if response:
            for key in [k for k in response.get_headers() if k in self._cookie_headers]:
                try:
                    cookie = cookie_dict[host]
                except KeyError, e:
                    cookie_dict[host] = SimpleCookie()
                    cookie = cookie_dict[host]
                try:
                    cookie.load(response.get_headers()[key])
                    self.get_core().get_output_manager().debug("Found cookie in %s\nValue=%s" % (host,cookie.output()))
                    
                except CookieError, e:
                    message = "Cookie from %s is invalid." % (host,)
                    self.get_core().get_output_manager().information(message)
        return response
    
    

