
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from httplib import HTTPSConnection
import re
import urllib
import time
import urlparse

from core.request_subsystem.RequestFilterPlugin import RequestFilterPlugin
from core.request_subsystem.Response import Response
from core.constants.option_types import INTEGER

class HTTPSSenderPlugin(RequestFilterPlugin):
    
    options = (
        RequestFilterPlugin.create_option('request.filter.HTTPSSenderPlugin', 'timeout', 10, INTEGER),
    )
    weight = 2 ** 31 - 1
    
    def __init__(self):
        RequestFilterPlugin.__init__(self)
        self._timeout = self.get_option('timeout').get_value()
    
    def request(self, request):
        parsed_url = urlparse.urlparse(request.get_url())
        if parsed_url.scheme == 'https':
            self._connection = HTTPSConnection(parsed_url.netloc, timeout=self._timeout)
            return getattr(self, '_' + request.get_method().lower())(request)
        else:
            return self._next(request)
    
    def _get(self, request):
        full_url = request.get_url() + '?' + urllib.urlencode(request.get_parameters())
        time_sent = time.time()
        self._connection.request('GET', full_url, headers=request.get_headers())
        response_con = self._connection.getresponse()
        time_arrived = time.time()
        response = Response(request,
                            response_con.status,
                            response_con.read(),
                            self._parse_headers(response_con.getheaders()),
                            time_arrived - time_sent)
        return response
    
    def _post(self, request):
        params = urllib.urlencode(request.get_parameters())
        time_sent = time.time()
        self._connection.request('POST', request.get_url(), body=params, headers=request.get_headers())
        response_con = self._connection.getresponse()
        time_arrived = time.time()
        response = Response(request,
                            response_con.status,
                            response_con.read(),
                            self._parse_headers(response_con.getheaders()),
                            time_arrived - time_sent)
        return response

    def _parse_headers(self, headers):
        headers_dict = {}
        for key, value in headers:
            headers_dict[key.upper()] = value
        return headers_dict